// Desktop Scroller.
// Copyright (C) 2011-2012 Marcos Diaz <diazmencia@gmail.com>.
// Copyright (C) 2012 Arnaud Bonatti <arnaud.bonatti@gmail.com>.
//
// Desktop Scroller is libre software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 2 of the License, or newer.
//
// You should have received a copy of the GNU General Public License along with
// this file. If not, see <http://www.gnu.org/licenses/>.

const Clutter					= imports.gi.Clutter
const Lang						= imports.lang
const Main						= imports.ui.main
const Mainloop					= imports.mainloop
const Meta						= imports.gi.Meta
const WorkspaceSwitcherPopup	= imports.ui.workspaceSwitcherPopup

const WIDTH_px	= 1
const WAITER_ms	= 456

function Scroller() { this._init() }

Scroller.prototype = {
	_init: function() {
		let y = Main.panel.actor.get_height() + Main.panel._leftCorner.actor.get_height()	
		this.actor = new Clutter.Rectangle({
			name: 'left-scroller',
			opacity: 0,
			reactive: true, 
			'x': 0, 'width' : WIDTH_px,
			'y': y, 'height': Main.layoutManager.primaryMonitor.height - y
		})
		
		this.actor._delegate = this
		this._timeoutId = null
		
		this.actor.connect('scroll-event', function(actor, event) {
			if (!actor._delegate._timeoutId) {
				// see js/ui/workspacesView.js:1042
				switch (event.get_scroll_direction()) {
				case Clutter.ScrollDirection.UP:
					actor._delegate._doScroll(Meta.MotionDirection.UP)
					break
				case Clutter.ScrollDirection.DOWN:
					actor._delegate._doScroll(Meta.MotionDirection.DOWN)
					break
				}
			}
		})
		
		Main.layoutManager.addChrome(this.actor, { /* visibleInFullscreen:true */ })
	},
	
	_doScroll: function(direction) {
		let newWs = Main.wm.actionMoveWorkspace(direction)
		
		// for scrolling more than one time (but not at the same time)
		this._timeoutId = Mainloop.timeout_add(WAITER_ms, Lang.bind(this, function() { this._timeoutId = null; return false }))
		
		// workspaceSwitcher popup, see js/ui/windowManager.js:575
		if (!Main.overview.visible) {
			if (Main.wm._workspaceSwitcherPopup == null) {
				Main.wm._workspaceSwitcherPopup = new WorkspaceSwitcherPopup.WorkspaceSwitcherPopup()				
				Main.wm._workspaceSwitcherPopup.connect('destroy', function() { Main.wm._workspaceSwitcherPopup = null })
			}
			// The 'workspace-switcher-group' takes all the screen, we should be on top to scroll more than one time
			Main.layoutManager.removeChrome(this.actor)
			Main.layoutManager.addChrome(this.actor, { /* visibleInFullscreen:true */ })
			
			Main.wm._workspaceSwitcherPopup.display(direction, newWs.index())
		}
	},
	
	destroy: function() {
		if (this._timeoutId) Mainloop.source_remove(this._timeoutId)
		this._timeoutId = null
		
		Main.layoutManager.removeChrome(this.actor)
		this.actor.destroy()
	}
}

let desktopscroller
function init() 	{ }
function enable()	{ desktopscroller = new Scroller() }
function disable()	{ desktopscroller.destroy()
					  desktopscroller = null }
